var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { AbstractRemote } from '@journeyapps/powersync-sdk-common';
export class WebRemote extends AbstractRemote {
    post(path, data, headers = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = yield this.buildRequest(path);
            const res = yield fetch(request.url, {
                method: 'POST',
                headers: Object.assign(Object.assign({}, headers), request.headers),
                body: JSON.stringify(data)
            });
            if (!res.ok) {
                throw new Error(`Received ${res.status} - ${res.statusText} when posting to ${path}: ${yield res.text()}}`);
            }
            return res.json();
        });
    }
    get(path, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = yield this.buildRequest(path);
            const res = yield fetch(request.url, {
                method: 'GET',
                headers: Object.assign(Object.assign({}, headers), request.headers)
            });
            if (!res.ok) {
                throw new Error(`Received ${res.status} - ${res.statusText} when getting from ${path}: ${yield res.text()}}`);
            }
            return res.json();
        });
    }
    postStreaming(path, data, headers = {}, signal) {
        return __awaiter(this, void 0, void 0, function* () {
            const request = yield this.buildRequest(path);
            /**
             * This abort controller will abort the fetch request if it has not resolved yet.
             * If the request has resolved, it will be used to close the stream.
             * Aborting the request after the stream is resolved and is in use seems to throw
             * unhandled exceptions on the window level.
             */
            const controller = new AbortController();
            let requestResolved = false;
            signal === null || signal === void 0 ? void 0 : signal.addEventListener('abort', () => {
                if (!requestResolved) {
                    // Only abort via the abort controller if the request has not resolved yet
                    controller.abort(signal.reason);
                }
            });
            const res = yield fetch(request.url, {
                method: 'POST',
                headers: Object.assign(Object.assign({}, headers), request.headers),
                body: JSON.stringify(data),
                signal: controller.signal,
                cache: 'no-store'
            }).catch((ex) => {
                // Handle abort requests which occur before the response resolves
                if (ex.name == 'AbortError') {
                    this.logger.warn(`Fetch request for ${request.url} has been aborted`);
                    return;
                }
                throw ex;
            });
            if (!res) {
                throw new Error('Fetch request was aborted before resolving.');
            }
            requestResolved = true;
            if (!res.ok || !res.body) {
                const text = yield res.text();
                console.error(`Could not POST streaming to ${path} - ${res.status} - ${res.statusText}: ${text}`);
                const error = new Error(`HTTP ${res.statusText}: ${text}`);
                error.status = res.status;
                throw error;
            }
            /**
             * The can-ndjson-stream does not handle aborted streams well.
             * This will intercept the readable stream and close the stream if
             * aborted.
             */
            const reader = res.body.getReader();
            signal === null || signal === void 0 ? void 0 : signal.addEventListener('abort', () => {
                // This will close the network request and read stream
                reader.cancel();
            });
            const outputStream = new ReadableStream({
                start(controller) {
                    processStream();
                    function processStream() {
                        return __awaiter(this, void 0, void 0, function* () {
                            while (!(signal === null || signal === void 0 ? void 0 : signal.aborted)) {
                                try {
                                    const { done, value } = yield reader.read();
                                    // When no more data needs to be consumed, close the stream
                                    if (done) {
                                        break;
                                    }
                                    // Enqueue the next data chunk into our target stream
                                    controller.enqueue(value);
                                }
                                catch (ex) {
                                    console.error(ex);
                                    break;
                                }
                            }
                            if (!(signal === null || signal === void 0 ? void 0 : signal.aborted)) {
                                // Close the downstream readable stream
                                reader.cancel();
                            }
                            controller.close();
                            reader.releaseLock();
                        });
                    }
                }
            });
            // Create a new response out of the intercepted stream
            return new Response(outputStream).body;
        });
    }
}
//# sourceMappingURL=WebRemote.js.map